@@ -39,6 +39,56 @@ def test_from_single_2d_quaternion():
     assert_array_almost_equal(r.as_quat(), expected_quat)
 
 
+def test_from_quat_scalar_first():
+    rng = np.random.RandomState(0)
+
+    r = Rotation.from_quat([1, 0, 0, 0], scalar_first=True)
+    assert_allclose(r.as_matrix(), np.eye(3), rtol=1e-15, atol=1e-16)
+
+    r = Rotation.from_quat(np.tile([1, 0, 0, 0], (10, 1)), scalar_first=True)
+    assert_allclose(r.as_matrix(), np.tile(np.eye(3), (10, 1, 1)),
+                    rtol=1e-15, atol=1e-16)
+
+    q = rng.randn(100, 4)
+    q /= np.linalg.norm(q, axis=1)[:, None]
+    for qi in q:
+        r = Rotation.from_quat(qi, scalar_first=True)
+        assert_allclose(np.roll(r.as_quat(), 1), qi, rtol=1e-15)
+
+    r = Rotation.from_quat(q, scalar_first=True)
+    assert_allclose(np.roll(r.as_quat(), 1, axis=1), q, rtol=1e-15)
+
+
+def test_as_quat_scalar_first():
+    rng = np.random.RandomState(0)
+
+    r = Rotation.from_euler('xyz', np.zeros(3))
+    assert_allclose(r.as_quat(scalar_first=True), [1, 0, 0, 0],
+                    rtol=1e-15, atol=1e-16)
+
+    r = Rotation.from_euler('xyz', np.zeros((10, 3)))
+    assert_allclose(r.as_quat(scalar_first=True),
+                    np.tile([1, 0, 0, 0], (10, 1)), rtol=1e-15, atol=1e-16)
+
+    q = rng.randn(100, 4)
+    q /= np.linalg.norm(q, axis=1)[:, None]
+    for qi in q:
+        r = Rotation.from_quat(qi)
+        assert_allclose(r.as_quat(scalar_first=True), np.roll(qi, 1),
+                        rtol=1e-15)
+
+        assert_allclose(r.as_quat(canonical=True, scalar_first=True),
+                        np.roll(r.as_quat(canonical=True), 1),
+                        rtol=1e-15)
+
+    r = Rotation.from_quat(q)
+    assert_allclose(r.as_quat(scalar_first=True), np.roll(q, 1, axis=1),
+                    rtol=1e-15)
+
+    assert_allclose(r.as_quat(canonical=True, scalar_first=True),
+                    np.roll(r.as_quat(canonical=True), 1, axis=1), rtol=1e-15)
+
+
 def test_from_square_quat_matrix():
     # Ensure proper norm array broadcasting
     x = np.array([
@@ -99,18 +149,29 @@ def test_quat_double_cover():
                     [0, 0, 0, -1], atol=2e-16)
 
 
-def test_malformed_1d_from_quat():
-    with pytest.raises(ValueError):
+def test_from_quat_wrong_shape():
+    # Wrong shape 1d array
+    with pytest.raises(ValueError, match='Expected `quat` to have shape'):
         Rotation.from_quat(np.array([1, 2, 3]))
 
-
-def test_malformed_2d_from_quat():
-    with pytest.raises(ValueError):
+    # Wrong shape 2d array
+    with pytest.raises(ValueError, match='Expected `quat` to have shape'):
         Rotation.from_quat(np.array([
             [1, 2, 3, 4, 5],
             [4, 5, 6, 7, 8]
             ]))
 
+    # 3d array
+    with pytest.raises(ValueError, match='Expected `quat` to have shape'):
+        Rotation.from_quat(np.array([
+            [[1, 2, 3, 4]],
+            [[4, 5, 6, 7]]
+            ]))
+
+    # 0-length 2d array
+    with pytest.raises(ValueError, match='Expected `quat` to have shape'):
+        Rotation.from_quat(np.array([]).reshape((0, 4)))
+
 
 def test_zero_norms_from_quat():
     x = np.array([
@@ -655,7 +716,7 @@ def test_as_euler_asymmetric_axes(seq_tuple, intrinsic):
     seq = "".join(seq_tuple)
     if intrinsic:
         # Extrinsic rotation (wrt to global world) at lower case
-        # intrinsinc (WRT the object itself) lower case.
+        # intrinsic (WRT the object itself) lower case.
         seq = seq.upper()
     rotation = Rotation.from_euler(seq, angles)
     angles_quat = rotation.as_euler(seq)
@@ -698,6 +759,7 @@ def test_as_euler_symmetric_axes(seq_tuple, intrinsic):
     test_stats(angles_mat - angles, 1e-15, 1e-13)
 
 
+@pytest.mark.thread_unsafe
 @pytest.mark.parametrize("seq_tuple", permutations("xyz"))
 @pytest.mark.parametrize("intrinsic", (False, True))
 def test_as_euler_degenerate_asymmetric_axes(seq_tuple, intrinsic):
@@ -724,6 +786,7 @@ def test_as_euler_degenerate_asymmetric_axes(seq_tuple, intrinsic):
     assert_array_almost_equal(mat_expected, mat_estimated)
 
 
+@pytest.mark.thread_unsafe
 @pytest.mark.parametrize("seq_tuple", permutations("xyz"))
 @pytest.mark.parametrize("intrinsic", (False, True))
 def test_as_euler_degenerate_symmetric_axes(seq_tuple, intrinsic):
@@ -751,6 +814,7 @@ def test_as_euler_degenerate_symmetric_axes(seq_tuple, intrinsic):
     assert_array_almost_equal(mat_expected, mat_estimated)
 
 
+@pytest.mark.thread_unsafe
 @pytest.mark.parametrize("seq_tuple", permutations("xyz"))
 @pytest.mark.parametrize("intrinsic", (False, True))
 def test_as_euler_degenerate_compare_algorithms(seq_tuple, intrinsic):
@@ -796,7 +860,7 @@ def test_as_euler_degenerate_compare_algorithms(seq_tuple, intrinsic):
     # Rotation of the form A/B/A are rotation around symmetric axes
     seq = "".join([seq_tuple[0], seq_tuple[1], seq_tuple[0]])
     if intrinsic:
-        # Extrinsinc rotation (wrt to global world) at lower case
+        # Extrinsic rotation (wrt to global world) at lower case
         # Intrinsic (WRT the object itself) upper case.
         seq = seq.upper()
 
@@ -821,6 +885,7 @@ def test_as_euler_degenerate_compare_algorithms(seq_tuple, intrinsic):
 def test_inv():
     rnd = np.random.RandomState(0)
     n = 10
+    # preserve use of old random_state during SPEC 7 transition
     p = Rotation.random(num=n, random_state=rnd)
     q = p.inv()
 
@@ -837,8 +902,8 @@ def test_inv():
 
 
 def test_inv_single_rotation():
-    rnd = np.random.RandomState(0)
-    p = Rotation.random(random_state=rnd)
+    rng = np.random.default_rng(146972845698875399755764481408308808739)
+    p = Rotation.random(rng=rng)
     q = p.inv()
 
     p_mat = p.as_matrix()
@@ -851,7 +916,7 @@ def test_inv_single_rotation():
     assert_array_almost_equal(res1, eye)
     assert_array_almost_equal(res2, eye)
 
-    x = Rotation.random(num=1, random_state=rnd)
+    x = Rotation.random(num=1, rng=rng)
     y = x.inv()
 
     x_matrix = x.as_matrix()
@@ -879,7 +944,7 @@ def test_single_identity_magnitude():
 
 def test_identity_invariance():
     n = 10
-    p = Rotation.random(n, random_state=0)
+    p = Rotation.random(n, rng=0)
 
     result = p * Rotation.identity(n)
     assert_array_almost_equal(p.as_quat(), result.as_quat())
@@ -890,7 +955,7 @@ def test_identity_invariance():
 
 def test_single_identity_invariance():
     n = 10
-    p = Rotation.random(n, random_state=0)
+    p = Rotation.random(n, rng=0)
 
     result = p * Rotation.identity()
     assert_array_almost_equal(p.as_quat(), result.as_quat())
@@ -919,15 +984,16 @@ def test_magnitude_single_rotation():
 
 
 def test_approx_equal():
-    rng = np.random.RandomState(0)
-    p = Rotation.random(10, random_state=rng)
-    q = Rotation.random(10, random_state=rng)
+    rng = np.random.default_rng(146972845698875399755764481408308808739)
+    p = Rotation.random(10, rng=rng)
+    q = Rotation.random(10, rng=rng)
     r = p * q.inv()
     r_mag = r.magnitude()
     atol = np.median(r_mag)  # ensure we get mix of Trues and Falses
     assert_equal(p.approx_equal(q, atol), (r_mag < atol))
 
 
+@pytest.mark.thread_unsafe
 def test_approx_equal_single_rotation():
     # also tests passing single argument to approx_equal
     p = Rotation.from_rotvec([0, 0, 1e-9])  # less than default atol of 1e-8
@@ -976,7 +1042,7 @@ def test_reduction_no_indices():
 
 def test_reduction_none_indices():
     result = Rotation.identity().reduce(return_indices=True)
-    assert type(result) == tuple
+    assert type(result) is tuple
     assert len(result) == 3
 
     reduced, left_best, right_best = result
@@ -985,10 +1051,10 @@ def test_reduction_none_indices():
 
 
 def test_reduction_scalar_calculation():
-    rng = np.random.RandomState(0)
-    l = Rotation.random(5, random_state=rng)
-    r = Rotation.random(10, random_state=rng)
-    p = Rotation.random(7, random_state=rng)
+    rng = np.random.default_rng(146972845698875399755764481408308808739)
+    l = Rotation.random(5, rng=rng)
+    r = Rotation.random(10, rng=rng)
+    p = Rotation.random(7, rng=rng)
     reduced, left_best, right_best = p.reduce(l, r, return_indices=True)
 
     # Loop implementation of the vectorized calculation in Rotation.reduce
@@ -1140,23 +1206,23 @@ def test_setitem_single():
 
 
 def test_setitem_slice():
-    rng = np.random.RandomState(seed=0)
-    r1 = Rotation.random(10, random_state=rng)
-    r2 = Rotation.random(5, random_state=rng)
+    rng = np.random.default_rng(146972845698875399755764481408308808739)
+    r1 = Rotation.random(10, rng=rng)
+    r2 = Rotation.random(5, rng=rng)
     r1[1:6] = r2
     assert_equal(r1[1:6].as_quat(), r2.as_quat())
 
 
 def test_setitem_integer():
-    rng = np.random.RandomState(seed=0)
-    r1 = Rotation.random(10, random_state=rng)
-    r2 = Rotation.random(random_state=rng)
+    rng = np.random.default_rng(146972845698875399755764481408308808739)
+    r1 = Rotation.random(10, rng=rng)
+    r2 = Rotation.random(rng=rng)
     r1[1] = r2
     assert_equal(r1[1].as_quat(), r2.as_quat())
 
 
 def test_setitem_wrong_type():
-    r = Rotation.random(10, random_state=0)
+    r = Rotation.random(10, rng=0)
     with pytest.raises(TypeError, match='Rotation object'):
         r[0] = 1
 
@@ -1180,12 +1246,12 @@ def test_n_rotations():
 
 
 def test_random_rotation_shape():
-    rnd = np.random.RandomState(0)
-    assert_equal(Rotation.random(random_state=rnd).as_quat().shape, (4,))
-    assert_equal(Rotation.random(None, random_state=rnd).as_quat().shape, (4,))
+    rng = np.random.default_rng(146972845698875399755764481408308808739)
+    assert_equal(Rotation.random(rng=rng).as_quat().shape, (4,))
+    assert_equal(Rotation.random(None, rng=rng).as_quat().shape, (4,))
 
-    assert_equal(Rotation.random(1, random_state=rnd).as_quat().shape, (1, 4))
-    assert_equal(Rotation.random(5, random_state=rnd).as_quat().shape, (5, 4))
+    assert_equal(Rotation.random(1, rng=rng).as_quat().shape, (1, 4))
+    assert_equal(Rotation.random(5, rng=rng).as_quat().shape, (5, 4))
 
 
 def test_align_vectors_no_rotation():
@@ -1198,9 +1264,9 @@ def test_align_vectors_no_rotation():
 
 
 def test_align_vectors_no_noise():
-    rnd = np.random.RandomState(0)
-    c = Rotation.random(random_state=rnd)
-    b = rnd.normal(size=(5, 3))
+    rng = np.random.default_rng(14697284569885399755764481408308808739)
+    c = Rotation.random(rng=rng)
+    b = rng.normal(size=(5, 3))
     a = c.apply(b)
 
     est, rssd = Rotation.align_vectors(a, b)
@@ -1235,8 +1301,8 @@ def test_align_vectors_rssd_sensitivity():
 
 def test_align_vectors_scaled_weights():
     n = 10
-    a = Rotation.random(n, random_state=0).apply([1, 0, 0])
-    b = Rotation.random(n, random_state=1).apply([1, 0, 0])
+    a = Rotation.random(n, rng=0).apply([1, 0, 0])
+    b = Rotation.random(n, rng=1).apply([1, 0, 0])
     scale = 2
 
     est1, rssd1, cov1 = Rotation.align_vectors(a, b, np.ones(n), True)
@@ -1248,17 +1314,17 @@ def test_align_vectors_scaled_weights():
 
 
 def test_align_vectors_noise():
-    rnd = np.random.RandomState(0)
+    rng = np.random.default_rng(146972845698875399755764481408308808739)
     n_vectors = 100
-    rot = Rotation.random(random_state=rnd)
-    vectors = rnd.normal(size=(n_vectors, 3))
+    rot = Rotation.random(rng=rng)
+    vectors = rng.normal(size=(n_vectors, 3))
     result = rot.apply(vectors)
 
     # The paper adds noise as independently distributed angular errors
     sigma = np.deg2rad(1)
     tolerance = 1.5 * sigma
     noise = Rotation.from_rotvec(
-        rnd.normal(
+        rng.normal(
             size=(n_vectors, 3),
             scale=sigma
         )
@@ -1372,7 +1438,7 @@ def test_align_vectors_near_inf():
     n = 100
     mats = []
     for i in range(6):
-        mats.append(Rotation.random(n, random_state=10 + i).as_matrix())
+        mats.append(Rotation.random(n, rng=10 + i).as_matrix())
 
     for i in range(n):
         # Get random pairs of 3-element vectors
@@ -1417,10 +1483,36 @@ def test_align_vectors_parallel():
     assert_allclose(R.apply(b[0]), a[0], atol=atol)
 
 
+def test_align_vectors_antiparallel():
+    # Test exact 180 deg rotation
+    atol = 1e-12
+    as_to_test = np.array([[[1, 0, 0], [0, 1, 0]],
+                           [[0, 1, 0], [1, 0, 0]],
+                           [[0, 0, 1], [0, 1, 0]]])
+    bs_to_test = [[-a[0], a[1]] for a in as_to_test]
+    for a, b in zip(as_to_test, bs_to_test):
+        R, _ = Rotation.align_vectors(a, b, weights=[np.inf, 1])
+        assert_allclose(R.magnitude(), np.pi, atol=atol)
+        assert_allclose(R.apply(b[0]), a[0], atol=atol)
+
+    # Test exact rotations near 180 deg
+    Rs = Rotation.random(100, rng=0)
+    dRs = Rotation.from_rotvec(Rs.as_rotvec()*1e-4)  # scale down to small angle
+    a = [[ 1, 0, 0], [0, 1, 0]]
+    b = [[-1, 0, 0], [0, 1, 0]]
+    as_to_test = []
+    for dR in dRs:
+        as_to_test.append([dR.apply(a[0]), a[1]])
+    for a in as_to_test:
+        R, _ = Rotation.align_vectors(a, b, weights=[np.inf, 1])
+        R2, _ = Rotation.align_vectors(a, b, weights=[1e10, 1])
+        assert_allclose(R.as_matrix(), R2.as_matrix(), atol=atol)
+
+
 def test_align_vectors_primary_only():
     atol = 1e-12
-    mats_a = Rotation.random(100, random_state=0).as_matrix()
-    mats_b = Rotation.random(100, random_state=1).as_matrix()
+    mats_a = Rotation.random(100, rng=0).as_matrix()
+    mats_b = Rotation.random(100, rng=1).as_matrix()
     for mat_a, mat_b in zip(mats_a, mats_b):
         # Get random 3-element unit vectors
         a = mat_a[0]
@@ -1571,8 +1663,8 @@ def test_slerp_call_scalar_time():
 
 
 def test_multiplication_stability():
-    qs = Rotation.random(50, random_state=0)
-    rs = Rotation.random(1000, random_state=1)
+    qs = Rotation.random(50, rng=0)
+    rs = Rotation.random(1000, rng=1)
     for q in qs:
         rs *= q * rs
         assert_allclose(np.linalg.norm(rs.as_quat(), axis=1), 1)
@@ -1580,7 +1672,7 @@ def test_multiplication_stability():
 
 def test_pow():
     atol = 1e-14
-    p = Rotation.random(10, random_state=0)
+    p = Rotation.random(10, rng=0)
     p_inv = p.inv()
     # Test the short-cuts and other integers
     for n in [-5, -2, -1, 0, 1, 2, 5]:
@@ -1616,14 +1708,14 @@ def test_pow():
 
 
 def test_pow_errors():
-    p = Rotation.random(random_state=0)
+    p = Rotation.random(rng=0)
     with pytest.raises(NotImplementedError, match='modulus not supported'):
         pow(p, 1, 1)
 
 
 def test_rotation_within_numpy_array():
-    single = Rotation.random(random_state=0)
-    multiple = Rotation.random(2, random_state=1)
+    single = Rotation.random(rng=0)
+    multiple = Rotation.random(2, rng=1)
 
     array = np.array(single)
     assert_equal(array.shape, ())
@@ -1675,13 +1767,24 @@ def test_as_euler_contiguous():
 
 
 def test_concatenate():
-    rotation = Rotation.random(10, random_state=0)
+    rotation = Rotation.random(10, rng=0)
     sizes = [1, 2, 3, 1, 3]
     starts = [0] + list(np.cumsum(sizes))
     split = [rotation[i:i + n] for i, n in zip(starts, sizes)]
     result = Rotation.concatenate(split)
     assert_equal(rotation.as_quat(), result.as_quat())
 
+    # Test Rotation input for multiple rotations
+    result = Rotation.concatenate(rotation)
+    assert_equal(rotation.as_quat(), result.as_quat())
+
+    # Test that a copy is returned
+    assert rotation is not result
+
+    # Test Rotation input for single rotations
+    result = Rotation.concatenate(Rotation.identity())
+    assert_equal(Rotation.identity().as_quat(), result.as_quat())
+
 
 def test_concatenate_wrong_type():
     with pytest.raises(TypeError, match='Rotation objects only'):
@@ -1690,19 +1793,16 @@ def test_concatenate_wrong_type():
 
 # Regression test for gh-16663
 def test_len_and_bool():
-    rotation_multi_empty = Rotation(np.empty((0, 4)))
     rotation_multi_one = Rotation([[0, 0, 0, 1]])
     rotation_multi = Rotation([[0, 0, 0, 1], [0, 0, 0, 1]])
     rotation_single = Rotation([0, 0, 0, 1])
 
-    assert len(rotation_multi_empty) == 0
     assert len(rotation_multi_one) == 1
     assert len(rotation_multi) == 2
     with pytest.raises(TypeError, match="Single rotation has no len()."):
         len(rotation_single)
 
     # Rotation should always be truthy. See gh-16663
-    assert rotation_multi_empty
     assert rotation_multi_one
     assert rotation_multi
     assert rotation_single
@@ -1710,7 +1810,7 @@ def test_len_and_bool():
 
 def test_from_davenport_single_rotation():
     axis = [0, 0, 1]
-    quat = Rotation.from_davenport(axis, 'extrinsic', 90, 
+    quat = Rotation.from_davenport(axis, 'extrinsic', 90,
                                    degrees=True).as_quat()
     expected_quat = np.array([0, 0, 1, 1]) / np.sqrt(2)
     assert_allclose(quat, expected_quat)
@@ -1731,17 +1831,17 @@ def test_from_davenport_one_or_two_axes():
     rot_dav = Rotation.from_davenport([ez], 'e', [np.pi/4])
     assert_allclose(rot.as_quat(canonical=True),
                     rot_dav.as_quat(canonical=True))
-    
+
     # Single rotation, two axes, axes.shape == (2, 3)
-    rot = Rotation.from_rotvec([np.array(ez) * np.pi/4, 
+    rot = Rotation.from_rotvec([np.array(ez) * np.pi/4,
                                 np.array(ey) * np.pi/6])
     rot = rot[0] * rot[1]
     rot_dav = Rotation.from_davenport([ey, ez], 'e', [np.pi/6, np.pi/4])
     assert_allclose(rot.as_quat(canonical=True),
                     rot_dav.as_quat(canonical=True))
-    
+
     # Two rotations, single axis, axes.shape == (3, )
-    rot = Rotation.from_rotvec([np.array(ez) * np.pi/6, 
+    rot = Rotation.from_rotvec([np.array(ez) * np.pi/6,
                                 np.array(ez) * np.pi/4])
     rot_dav = Rotation.from_davenport([ez], 'e', [np.pi/6, np.pi/4])
     assert_allclose(rot.as_quat(canonical=True),
@@ -1784,6 +1884,7 @@ def test_as_davenport():
             assert_allclose(angles_dav, angles)
 
 
+@pytest.mark.thread_unsafe
 def test_as_davenport_degenerate():
     # Since we cannot check for angle equality, we check for rotation matrix
     # equality
@@ -1830,7 +1931,7 @@ def test_compare_from_davenport_from_euler():
                 seq = seq.upper()
             eul = Rotation.from_euler(seq, angles)
             dav = Rotation.from_davenport(ax, order, angles)
-            assert_allclose(eul.as_quat(canonical=True), dav.as_quat(canonical=True), 
+            assert_allclose(eul.as_quat(canonical=True), dav.as_quat(canonical=True),
                             rtol=1e-12)
 
     # asymmetric sequences
